<?php
/*
 * $Id: jleagueAPI.class.php 459 2008-02-22 03:52:52Z koichi $
 */
require_once 'PEAR.php';
require_once 'HTTP.php';
require_once 'Cache/Lite.php';

require_once('jleagueclub.class.php');

/**
 * J.Leagueの試合予定を返すAPIクラス
 */
class jleagueAPI {
	/**
	 * URL取得用の変換キー
	 */
	protected $leaguekeys = array(
			'j1' => '010001',
			'j1-1' => '010001',
			'j1-2' => '010002',
			'j2' => '020003',
			'ync' => '002000',
		);

	/**
	 * リーグ
	 */
	protected $league = null;
	public function getLeague()
	{
		return $this->league;
	}
	
	protected $year = null;
	/**
	 * 年
	 */
	public function getYear()
	{
		return $this->year;
	}

	/**
	 * キャッシュのディレクトリ
	 */
	protected $cacheDir = "/tmp/cache/";
	
	/**
	 * キャッシュオブジェクト
	 */
	protected $cache;

	/**
	 * 結果
	 */
	protected $result;
	
	/**
	 * コンストラクタ
	 */
	function __construct($league, $year, $cacheDir = NULL)
	{
		$this->league = $league;
		$this->year = $year;
		if($cacheDir) {
			$this->cacheDir = $cacheDir;
		}
	}
	
	/**
	 * URLを生成する
	 */
	private function getURL()
	{
		$url = 'http://www.j-league.or.jp/';
		$url .= 'SS/jpn/news/';
		$url .= $this->year;
		$url .= $this->leaguekeys[$this->league];
		$url .= '_W0501_J.html';
		
		return $url;
	}
	
	/**
	 * パースする
	 */
	public function execParse()
	{
		// キャッシュの設定 60秒x60分x24時間x60日保存する
		$options = array(
			'cacheDir' => $this->cacheDir,
			'lifeTime' => 60 * 60 * 24 * 60
		);
		
		// Cache_Liteを生成する。
		$this->cache = new Cache_Lite($options);
		if(!$this->cache) {
			print "キャッシュが生成できていない";
			return;
		}
		// キャッシュキーを生成
		$cachekey = "jleagueAPI_" . $this->year . "_" . $this->league;
		
		// URLを取得する。
		$url = $this->getURL();
		
		// ヘッダーを取得
		$result = HTTP::head($url);
		if(PEAR::isError($result)) {
			echo "error: " . $result->getMessage();
		}
		else if($result['response_code'] != "200") {
			// Responseが200以外はエラーにする。
			$this->result['http_error'] = $result;
			$this->result['count'] = 0;
			return false;
		}
		
		// キャッシュを取得
		$data = $this->cache->get($cachekey);
		if(!$data) { // キャッシュがなかった場合
			// HTMLを取得する。
			$html = file_get_contents($url);
			// UTF-8に変換
			mb_convert_variables('UTF-8', 'Shift_JIS', $html);

			// loadHTMLの文字化け対策
			$html = str_replace('<BODY>', '<HEAD><META http-equiv="content-type" content="text/html; charset=UTF-8" /></HEAD><BODY>', $html);
			// キャッシュに保存
			$this->cache->save($html, $cachekey);
		}
		else { // キャッシュがあった場合
			$html = $data;
		}	
		
		// DOMdocumentを生成
		$doc = new DOMdocument();
		// HTMLを読み込む
		$doc->loadHTML($html);
		
		// DOMXPathを生成
		$xpath = new DOMXPath($doc);
		// 予定表部分を取得
		$entries = $xpath->query("//center/table[@border='0']/tr");
		
		$secdata = array(); // 節毎のデータ
		$sec = NULL; // 節
		$date = NULL; // 日付
		$match = NULL; // 試合毎のデータ
		foreach($entries as $node) {
			// 横線はスルー
			$linepath = $xpath->query("td[@colspan='8']", $node);
			if($linepath->length) {
				continue;
			}
						
			// 節
			$secpath = $xpath->query("td[@width='101']/font[@size='+0']", $node);
			if($secpath->length) {
				$secvalue = trim($secpath->item(0)->nodeValue);
				if($secvalue != "") {
					if($sec == NULL || $sec['sec'] <> $secvalue) {
						// 節が変わったので、節毎のデータを収納
						if($sec <> NULL) {
							$secdata[] = $sec;
						}
						// 節毎のデータを初期化
						$sec = array();
						$sec['sec'] = $secvalue;
						$sec['match'] = array();
					}
				}
			}

			// 日ごとのデータをを初期化
			$data = array();
			// 開催日
			if($this->league == 'ync') {
				// ナビスコカップの場合
				$datepath = $xpath->query("td[@width='90']/font[@size='+0']", $node);
			}
			else {
				$datepath = $xpath->query("td[@width='37']/font[@size='+0']", $node);
			}
			if($datepath->length) {
				$datevalue = trim($datepath->item(0)->nodeValue);
				if($datevalue != "" && ereg("[0-9]{2}\/[0-9]{2}", $datevalue)) {
					// 日付だったら
					if($date != $datevalue) {
						// 日付が変わった
						$date = $datevalue;
					}
				}
			}

			// キックオフ時間
			$kickoffpath = $xpath->query("td[@width='42']/font[@size='+0']", $node);
			if($kickoffpath->length) {
				$kickofftime = trim($kickoffpath->item(0)->nodeValue);
			}
			
			// ホーム
			$homepath = $xpath->query("td[@width='53']/font[@size='+0']", $node);
			if($homepath->length) {
				$home = trim($homepath->item(0)->nodeValue);
			}

			// 処理しない場合
			if((($datevalue != "" || !ereg("[0-9]{2}\/[0-9]{2}", $datevalue)) && $home == "") || ($home == "ホーム")) {
				continue;
			}

			// ここで、日付、キックオフ時間、ホームを設定する。
			$data['date'] = $date; 
			$data['kickofftime'] = ereg("[0-9]{2}:[0-9]{2}", $kickofftime)?$kickofftime:'';
			$data['home'] = $home;
			
			// スコア
			$scorepath = $xpath->query("td[@width='62' and @align='center']/font[@size='+0']", $node);
			if($scorepath->length) {
				$score = trim($scorepath->item(0)->nodeValue);
				$data['score'] = $score!='vs'?$score:null; // スコアが設定されていたら収納
			}
			
			// アウェイ
			$awaypath = $xpath->query("td[@width='69']/font[@size='+0']", $node);
			if($awaypath->length) {
				$data['away'] = $awaypath->item(0)->nodeValue;
			}
			// 競技場
			$placepath = $xpath->query("td[@width='62' and @align='left']/font[@size='+0']", $node);
			if($placepath->length) {
				$data['place'] = $placepath->item(0)->nodeValue;
			}
			// 備考
			$notepath = $xpath->query("td[@width='275']/font[@size='+0']", $node);
			if($notepath->length) {
				$data['note'] = $notepath->item(0)->nodeValue;
			}
			
			// 試合後とに配列へ代入
			$sec['match'][] = $data;
		}
		
		// 最後の試合を処理
		if(count($sec['match']) > 0) {
			$secdata[] = $sec;
		}
		
		$this->result['year'] = $this->year;
		$this->result['league'] = $this->league;
		$this->result['count'] = count($secdata);
		$this->result['sec'] = $secdata;
		return true;
	}
	
	/**
	 * 結果を返す
	 */
	public function getResult()
	{
		return $this->result;
	}
	
	/**
	 * JSON形式で返す
	 */
	public function getJSON()
	{
		// JSONに変換して返す
		return json_encode($this->result);
	}
	
	/**
	 * XML形式で返す
	 */
	public function getXML()
	{
		require 'XML/Serializer.php';
	
		// XML_Serializerの設定	
		$options = array(
			"mode" => "simplexml" ,
			"indent"    => "\t",
			"linebreak" => "\n",
			"typeHints" => false,
			"addDecl"   => true,
			"encoding"  => "UTF-8",
			"rootName"  => "JLeagueSchedule",
			"rootAttributes" => array('version' => '1.0'),
			"defaultTagName" => "item",
			"attributesArray" => "_attributes"
		);
		
		// XML_Serializerを生成
		$serializer = &new XML_Serializer($options);
		
		// 結果をXMLへ
		$serializer->serialize($this->result);
		
		// 変換後の値を返す
		return $serializer->getSerializedData();
	}
}
?>